// $Id: userdir.c,v 1.3 2000/02/05 20:04:32 siefca Exp $
#include "userdir.h"
    
static char *nonulhome = "";

/******************************************************************************/
    
int getusercfv (char *variable, 
		    char *home_dir, 
		    char *user_dir, 
		    size_t stringsize)
{
	struct stat st;
	char *d;
	
	bzero (user_dir, stringsize);
	if (variable == NULL || *variable == '\0') return (-1);
	if (home_dir == NULL) home_dir = nonulhome;
	d = getenv (variable);	
	if (d == NULL || *d == '\0') return (-1);
	if (*d == '/') 
	    {
	    strncpy (user_dir, d, stringsize-1);
	    }
	else
	    {
	    snprintf (user_dir, stringsize-1, "%s/%s", home_dir, d);
	    }
	    
	if (stat(user_dir,&st) != -1 && S_ISDIR(st.st_mode)) 
	    {
	    return (0);
	    }
	
	return (-1);
}

/******************************************************************************/

int detectcfv(struct cfv *CFV) {
	if (CFV->subname == NULL) CFV->subname = nonulhome;
	if (CFV->home_dir == NULL) CFV->home_dir = getenv ("HOME");
	if (CFV->home_dir == NULL) CFV->home_dir = CFV->home_scd;
	if (CFV->home_dir == NULL) return (-1);
	return 0;
}

/******************************************************************************/

int preparemain(struct cfv *CFV) {
	char *p;
	struct stat st;
	if ((p = strrchr(CFV->result, '/')) && *(p+1) == '\0') 
	    p = '\0';
	snprintf ((rindex(CFV->result,'\0')), MAXPATHLEN-1, "/%s", 
		  CFV->subname);
	if (stat( CFV->result,&st) != -1 
	     && CFV->mode ? 
		S_ISDIR(st.st_mode) : 
		S_ISREG(st.st_mode)) /* have file or dir */
	    return (1);
	return 0;
}

/******************************************************************************/

int preparehome(struct cfv *CFV) {
	struct stat st;
    	if (CFV->home_dir == NULL || *(CFV->home_dir) == '\0') return (-1);
	if (CFV->prefix == NULL && CFV->suffix == NULL) return (-1);
	snprintf (CFV->result, MAXPATHLEN-1, "%s/%s%s%s", 
		       CFV->home_dir, 
		       CFV->prefix? CFV->prefix:"",
		       CFV->subname,
		       CFV->suffix? CFV->suffix:"");
	if (stat(CFV->result,&st) != -1 
		 && CFV->mode ? 
		    S_ISDIR(st.st_mode) : 
		    S_ISREG(st.st_mode)) /* have file or dir */
		return (2);
	return (-1);
}

/******************************************************************************/

int usercfv (struct cfv *CFV) {
	int gr;
	gr = detectcfv(CFV);
	if (gr) return gr;
	/* get environment variable */
	gr = getusercfv (CFV->variable, 
			CFV->home_dir, 
			CFV->result,
			MAXPATHLEN);
	if (gr != -1) 	/* have main directory */
	    {
		gr = preparemain(CFV);
		if (gr) return gr;
	    }
	/* don't have main directory or a proper variable set */
	gr = preparehome(CFV);
	return gr;
}
	
/******************************************************************************/

/* If directory or file doesn't exist returns the best one wich may be created */
int notnullusercfv (struct cfv *CFV) {
	int gr;
	if ((usercfv(CFV)) == -1) {
	    gr = detectcfv(CFV);
	    if (gr) return gr;	/* remember, it works for errors! */
	    /* get the environment variable */
	    gr = getusercfv (CFV->variable, 
			    CFV->home_dir, 
			    CFV->result,
			    MAXPATHLEN);
	    if (gr != -1) 	/* have main directory */
		{
		    gr = preparemain(CFV);
		    return 0;
		}
	    /* don't have main directory or a proper variable set */
	    gr = preparehome(CFV);
	    return 0;
 	}
	return 0;	
}
    
/******************************************************************************/

FILE *maynullfopencfv (struct cfv *CFV, const char *mode)
    {
    FILE *cfvfile = NULL;
    
    CFV->mode = M_REGULAR_FILE;
    if ((usercfv (CFV)) == -1) return (NULL);	
    cfvfile = fopen (CFV->result, mode);
    return (cfvfile);
    }
/******************************************************************************/

FILE *notnullfopencfv (struct cfv *CFV, const char *mode)
    {
    FILE *cfvfile = NULL;
    
    CFV->mode = M_REGULAR_FILE;
    if ((notnullusercfv (CFV)) == -1) return (NULL);	
    cfvfile = fopen (CFV->result, mode);
    return (cfvfile);
    }

/******************************************************************************/
    
FILE *fopencfv (struct cfv *CFV, const char *mode)
    {
    FILE *cfvfile = NULL;

    if (cfvfile == NULL && (strpbrk(mode, "wa")) != NULL) /* create in best location */
	cfvfile = notnullfopencfv (CFV, mode);
    else 
	cfvfile = maynullfopencfv (CFV, mode);
    
    return (cfvfile);
    }

/******************************************************************************/